//LTC2321 Verilog Driver for Altera Cyclone III FPGA
//Data format from ADC is CMOS
//Input clock rate is 29 times the output clock rate for the 12 bit part,
//31 times the output clock rate for the 14 bit part
//and 32 times the output clock rate for the 16 bit part.
//Note that DB[] bits are read on ch 1 of PSCOPE, and DA[] bits are read on ch 2
//The inputs BITSEL1, BITSEL2 select between 12, 14, & 16 bit formats as follows:
//  BITSEL1 = 0, BITSEL2 = X: select 12 bits
//  BITSEL1 = 1, BITSEL2 = 0: select 14 bits
//  BITSEL1 = 1, BITSEL2 = 1: select 16 bits
//DJS 5_9_16

`timescale 1ps / 1ps
module LTC2321_cmos
(	
	CLKIN,
	CSB,
	SCK,SCK_GND,
	CLKOUT,CLKOUT_GND,
	CNVCLK,
	SDO1,SDO2,SDO1_GND,SDO2_GND,
	DB0,DB1,DB2,DB3,DB4,DB5,DB6,DB7,DB8,DB9,DB10,DB11,DB12,DB13,DB14,DB15,
	DA0,DA1,DA2,DA3,DA4,DA5,DA6,DA7,DA8,DA9,DA10,DA11,DA12,DA13,DA14,DA15,
	BITSEL1,BITSEL2
);

input BITSEL1, BITSEL2;
input CLKIN;					//Clock in from user @ 58/62/64 MHz
input	CLKOUT,CLKOUT_GND;	//CLKOUT signal from ADC
									//Data appeaars 0.5nsec after each falling edge
input	SDO1,SDO1_GND;			//Data out CH1 from ADC
input SDO2,SDO2_GND;			//Data out CH2 from ADC

output CNVCLK;				//Latch signal from FPGA to DC890 on falling edge
output CSB;					//Convert Start on rising edge							
output SCK,SCK_GND;		//SCK from FPGA to ADC
output DB0,DB1,DB2,DB3,DB4,DB5,DB6,DB7,DB8,DB9,DB10,DB11,DB12,DB13,DB14,DB15;  //Data outputs to DC890
output DA0,DA1,DA2,DA3,DA4,DA5,DA6,DA7,DA8,DA9,DA10,DA11,DA12,DA13,DA14,DA15;  //Data outputs to DC890

reg SCK_GND;
reg [15:0] DA;
reg [15:0] DB;
reg [5:0] num_bits;

wire CLKIN;
wire CLKOUT,CLKOUT_GND;
wire CSB;																																		
wire CNVCLK;														
wire sck_gate;
wire DB0,DB1,DB2,DB3,DB4,DB5,DB6,DB7,DB8,DB9,DB10,DB11,DB12,DB13,DB14,DB15;
wire DA0,DA1,DA2,DA3,DA4,DA5,DA6,DA7,DA8,DA9,DA10,DA11,DA12,DA13,DA14,DA15;						
wire SDO1,SDO1_GND,SDO2,SDO2_GND;
wire [15:0] dataB;
wire [15:0] dataA;

initial
  SCK_GND = 0;
  
always @ (CLKIN)  //decode the number of bits
begin
  if(BITSEL1 == 1)
    begin
      if(BITSEL2 == 1)  //16 bits
			num_bits = 5'd16;
      else  //14 bits
			num_bits = 5'd16;	//note the code has to clock out 16 bits for the 14 bit part
	 end
  else  //12 bits
	num_bits = 5'd14;	//note the code has to clock out 14 bits for the 12 bit part
end
  
sync_pll	synchronize
(
	.inclk0(CLKIN),
	.c0(clk_64),
	.c1(clk_128),
	.c2(sync)
);

//generate the 'SCK' signal
altddioout	sckddr 
( 
	.datain_h(~sck_gate),
	.datain_l(1'b1),
	.outclock(clk_64),
	.dataout(SCK)
);

//generate the timing pulses
pulse_generator pulses
(
	.clk(clk_128),
	.sync(sync),
	.csb(CSB),
	.cnvclk(CNVCLK),
	.sck_gate(sck_gate),
	.rx_reset(rx_reset),
	.num_bits(num_bits)
);

// Receive serial data from ADC and converts it to parallel format
deserializer rx_A
(
	.rx_reset(rx_reset),
	.clk(CLKOUT),
	.sdo(SDO2),						  
	.parallel_data(dataA),
	.num_bits(num_bits)
);

// Receive serial data from ADC and converts it to parallel format
deserializer rx_B
(
	.rx_reset(rx_reset),
	.clk(CLKOUT),
	.sdo(SDO1),					  
	.parallel_data(dataB),
	.num_bits(num_bits)
);

assign DA0 = DA[0];
assign DA1 = DA[1];
assign DA2 = DA[2];
assign DA3 = DA[3];
assign DA4 = DA[4];
assign DA5 = DA[5];
assign DA6 = DA[6];
assign DA7 = DA[7];
assign DA8 = DA[8];
assign DA9 = DA[9];
assign DA10 = DA[10];
assign DA11 = DA[11];
assign DA12 = DA[12];
assign DA13 = DA[13];
assign DA14 = DA[14];
assign DA15 = DA[15];

assign DB0 = DB[0];
assign DB1 = DB[1];
assign DB2 = DB[2];
assign DB3 = DB[3];
assign DB4 = DB[4];
assign DB5 = DB[5];
assign DB6 = DB[6];
assign DB7 = DB[7];
assign DB8 = DB[8];
assign DB9 = DB[9];
assign DB10 = DB[10];
assign DB11 = DB[11];
assign DB12 = DB[12];
assign DB13 = DB[13];
assign DB14 = DB[14];
assign DB15 = DB[15];

//load the parallel data from the channels onto the 16 bit buses
always @ (posedge CNVCLK)
begin
    DA = dataA; 
	 DB = dataB;
end

endmodule